/*
    6-20-06
    Nathan Seidle
    nathan@sparkfun.com
    Copyright Spark Fun Electronics 2006
    
    Software SPI control
    PIC 16F88
    
    Reading and writing to LIS3LV02DQ
    
    This is a very powerful, configurable, and precise triple axis accelerometer.
    
    At first look, the interface can be daunting, but once things are running,
    this is an impressive little sensor!
    
*/
#define Clock_8MHz
#define Baud_9600

#include "c:\Global\Code\C\16F88.h"  // device dependent interrupt definitions

#pragma origin 4

#define STATUS_LED PORTB.3

#define INT PORTA.0 //This is interrupt, not used, read as input and leave floating
#define SDO PORTA.1 //This is output from the LIS into the PIC
#define SDI PORTA.2 //This is input into the LIS from the PIC
#define SCL PORTB.0 //This is a clock into the LIS from the PIC
#define CS  PORTB.1 //This is input into LIS from PIC - Low for SPI mode
#define CK  PORTB.4 //Not used - pull low to ground

void boot_up(void);

uns8 spi_comm(uns8 outgoing_byte);
uns8 read_register(uns8 register_name);
void write_register(uns8 register_name, uns8 register_value);
void LIS3LV02DQ_Test(void);

void delay_ms(uns16 x);
uns8 bin2Hex(char x);
void printf(const char *nate, int16 my_byte);

void main(void)
{
    uns8 out_byte, in_byte, choice;
    
    boot_up(); //Init ports and settings
        
    
    CS = 1; //Unselect LIS
    CK = 0;
        
    STATUS_LED = 0;

    in_byte = read_register(0x0F); //Read register WHO_AM_I
    printf("WAI=%h ", in_byte);

    in_byte = read_register(0x0F); //Read register WHO_AM_I
    printf("WAI=%h ", in_byte);

    write_register(0x20, 0b.1000.0111); //Power on device, enable all axis, and turn off self test - CTRL_REG1

    while(1)
    {
        STATUS_LED ^= 1;

        if(RCIF)
        {
            choice = RCREG;
            
            if(choice == 'x') break;
        }
        
        int16 x_val, y_val, z_val;
        x_val.low8 = read_register(0x28); //Read outx_l
        x_val.high8 = read_register(0x29); //Read outx_h
        printf("x=%d ", x_val);

        y_val.low8 = read_register(0x2A); //Read outy_l
        y_val.high8 = read_register(0x2B); //Read outy_h
        printf("y=%d ", y_val);

        z_val.low8 = read_register(0x2C); //Read outz_l
        z_val.high8 = read_register(0x2D); //Read outz_h
        printf("z=%d ", z_val);

        printf("\n\r", 0);
        
        delay_ms(50);
    }

    while(1);


}//End Main


void boot_up(void)
{
    //OSCCON = 0b.0111.0000; //Setup internal oscillator for 8MHz
    //while(OSCCON.2 == 0); //Wait for frequency to stabilize

    //Setup Ports
    ANSEL = 0b.0000.0000; //Turn off A/D

    //Read RA0, RA1
    PORTA = 0b.0000.0000;
    TRISA = 0b.0000.0011;  //0 = Output, 1 = Input

    //Output RB0, RB1, RB4
    PORTB = 0b.0000.0000;
    TRISB = 0b.0000.0100;   //0 = Output, 1 = Input RX on RB2

    //Setup the hardware UART module
    //=============================================================
    SPBRG = 51; //8MHz for 9600 inital communication baud rate
    //SPBRG = 129; //20MHz for 9600 inital communication baud rate

    TXSTA = 0b.0010.0100; //8-bit asych mode, high speed uart enabled
    RCSTA = 0b.1001.0000; //Serial port enable, 8-bit asych continous receive mode
    //=============================================================
}

//Sends read command to LIS
uns8 read_register(uns8 register_name)
{
    uns8 in_byte;
    
    register_name |= 0b.1000.0000; //Set D7 to 1 for read mode

    CS = 0; //Select LIS

    in_byte = spi_comm(register_name); //SPI read 6-bit register from LIS
    //printf("In1 = %h ", in_byte);
    
    //in_byte is nothing, we need to clock in another 8 bits
    in_byte = spi_comm(0x00); //Send nothing, but we should get back the register value
    
    CS = 1;
    
    return(in_byte);
}

//Sends a write command to LIS
void write_register(uns8 register_name, uns8 register_value)
{
    uns8 in_byte;
    
    register_name &= 0b.0111.1111; //Clear D7 to 0 for write mode

    CS = 0; //Select LIS

    in_byte = spi_comm(register_name); //SPI read 6-bit register from LIS
    //printf("In1 = %h ", in_byte);
    
    //in_byte is nothing, we need to clock in the value to write
    in_byte = spi_comm(register_value); //Send nothing, but we should get back the register value
    
    CS = 1;
    
    //Return nothing
}

//Basic SPI send and receive
uns8 spi_comm(uns8 outgoing_byte)
{
    uns8 incoming_byte, x;

    SCL = 1; //Pull SPI clock high
    for(x = 0 ; x < 8 ; x++)
    {
        SDI = outgoing_byte.7; //Put bit on SPI data bus
        outgoing_byte <<= 1; //Rotate byte 1 to the left

        SCL = 0; //Toggle the SPI clock
        SCL = 1;

        incoming_byte <<= 1; //Rotate byte 1 to the left
        incoming_byte.0 = SDO; //Read bit on SPI data bus
    }
    
    return(incoming_byte);
}

//General short delay
void delay_ms(uns16 x)
{

    //Clocks out at 1006us per 1ms
    uns8 y, z;
    for ( ; x > 0 ; x--)
        for ( y = 0 ; y < 4 ; y++)
            for ( z = 0 ; z < 69 ; z++);
}

//Sends nate to the Transmit Register
void putc(uns8 nate)
{
    while(TXIF == 0);
    TXREG = nate;
}

uns8 getc(void)
{
    while(RCIF == 0);
    return (RCREG);
}    

uns8 scanc(void)
{
    uns16 counter = 0;
    
    //CREN = 0;
    //CREN = 1;
    
    RCIF = 0;
    while(RCIF == 0)
    {
        counter++;
        if(counter == 1000) return 0;
    }
    
    return (RCREG);
}   

//Returns ASCII Decimal and Hex values
uns8 bin2Hex(char x)
{
   skip(x);
   #pragma return[16] = "0123456789ABCDEF"
}

//Prints a string including variables
void printf(const char *nate, int16 my_byte)
{
  
    uns8 i, k, m, temp;
    uns8 high_byte = 0, low_byte = 0;
    uns8 y, z;
    
    uns8 decimal_output[5];
    
    for(i = 0 ; ; i++)
    {
        //delay_ms(3);
        
        k = nate[i];

        if (k == '\0') 
            break;

        else if (k == '%') //Print var
        {
            i++;
            k = nate[i];

            if (k == '\0') 
                break;
            else if (k == '\\') //Print special characters
            {
                i++;
                k = nate[i];
                
                putc(k);
                

            } //End Special Characters
            else if (k == 'b') //Print Binary
            {
                for( m = 0 ; m < 8 ; m++ )
                {
                    if (my_byte.7 == 1) putc('1');
                    if (my_byte.7 == 0) putc('0');
                    if (m == 3) putc(' ');
                    
                    my_byte = my_byte << 1;
                }
            } //End Binary               
            else if (k == 'd') //Print Decimal
            {
                //Print negative sign and take 2's compliment
                
                if(my_byte < 0)
                {
                    putc('-');
                    my_byte *= -1;
                }
                
                
                if (my_byte == 0)
                    putc('0');
                else
                {
                    //Divide number by a series of 10s
                    for(m = 4 ; my_byte > 0 ; m--)
                    {
                        temp = my_byte % (uns16)10;
                        decimal_output[m] = temp;
                        my_byte = my_byte / (uns16)10;               
                    }
                
                    for(m++ ; m < 5 ; m++)
                        putc(bin2Hex(decimal_output[m]));
                }
    
            } //End Decimal
            else if (k == 'h') //Print Hex
            {
                //New trick 3-15-04
                putc('0');
                putc('x');
                
                if(my_byte > 0x00FF)
                {
                    putc(bin2Hex(my_byte.high8 >> 4));
                    putc(bin2Hex(my_byte.high8 & 0b.0000.1111));
                }

                putc(bin2Hex(my_byte.low8 >> 4));
                putc(bin2Hex(my_byte.low8 & 0b.0000.1111));

                /*high_byte.3 = my_byte.7;
                high_byte.2 = my_byte.6;
                high_byte.1 = my_byte.5;
                high_byte.0 = my_byte.4;
            
                low_byte.3 = my_byte.3;
                low_byte.2 = my_byte.2;
                low_byte.1 = my_byte.1;
                low_byte.0 = my_byte.0;
        
                putc('0');
                putc('x');
            
                putc(bin2Hex(high_byte));
                putc(bin2Hex(low_byte));*/
            } //End Hex
            else if (k == 'f') //Print Float
            {
                putc('!');
            } //End Float
            else if (k == 'u') //Print Direct Character
            {
                //All ascii characters below 20 are special and screwy characters
                //if(my_byte > 20) 
                    putc(my_byte);
            } //End Direct
                        
        } //End Special Chars           

        else
            putc(k);
    }    
}